/**
 * Gumowski-Mira Demo
 * 
 * The Gumowski-Mira equation was developed at the CERN research centre in 1980 
 * by I. Gumowski and C. Mira to calculate the trajectories of sub-atomic particles.
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 * 
 */

package {
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.StageQuality;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.ui.Keyboard;
	import flash.utils.*;
	import com.subblue.utils.*;
	
	public class Mira extends Sprite
	{
		static public const RENDERPASS_COMPLETE:String = "renderpassComplete";
		
		private var bitmap:Bitmap;
		private var canvas:BitmapData;
		private var maxIterations:int = 50000;
		private var iterations:int = maxIterations;
		private var maxPasses:int = 10;
		private var passes:int = 0;
		private var ox:int;
		private var oy:int;
		private var x0:Number;
		private var y0:Number;
		private var interactiveColor:uint = 0xFF000000;
		private var finalColor:uint = 0x010000000;
		
		private var _color:uint;					// Colour of plotted point
		private var _exportUrl:String = "/export";
		private var _exportExt:String = "png";
		
		private var a:Number = 0.000001; //0.00001;
		private var b:Number = 0.05;
		private var mu:Number = -0.084289;
		private var muInc:Number = 0.00001;
		private var _zoom:Number = 7;
		private var _paused:Boolean  = false;
		
		
		public function Mira()
		{
			stage.scaleMode = "noScale";
			stage.align = "TL";
			stage.quality = StageQuality.BEST;
			
			stage.addEventListener("resize", init);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMoveListener);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, mouseDownListener);
			stage.addEventListener(MouseEvent.MOUSE_UP, mouseUpListener);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownListener);
			init();
		}	
		
		
		private function init(e:Event = null):void
		{
			if (stage.stageWidth)
			{
				
				canvas = new BitmapData(stage.stageWidth, stage.stageHeight, true, 0x00FFFFFF);
				bitmap = new Bitmap(canvas, "auto", true);
				ox = stage.stageWidth / 2 - 20;
				oy = stage.stageHeight / 2;
				_color = finalColor;
				
				if (contains(bitmap)) removeChild(bitmap);
			
				addChild(bitmap);
				prepareCanvas();
				
				stage.addEventListener(Event.ENTER_FRAME, updateCanvas);
			} else {
				setTimeout(init, 100);
			}
		}
		
		
		private function prepareCanvas():void
		{
			x0 = 5;
			y0 = 5;
			passes = 0;
			_paused = false;
			trace("mu:", mu, "a:", a, "zoom:", _zoom);
			canvas.fillRect(new Rectangle(0, 0, canvas.width, canvas.height), 0xFFFFFFFF);
			canvas.fillRect(new Rectangle(0, 0, canvas.width, canvas.height), 0x00FFFFFF);
			updateCanvas();
		}
		
		
		private function updateCanvas(e:Event = null):void
		{
			if (!_paused) 
			{
				passes++;
				renderCanvas();
				if (passes > maxPasses) 
				{
					_paused = true;
					passes = 0;
				}
			}
		}
		
		
		private function renderCanvas():void
		{
			canvas.lock();
			
			var xn:Number, yn:Number;
			var px:Number, py:Number;
			
			for (var i:int = 0; i < iterations; i++)
			{
				xn = y0 + a * (1 - b * y0 * y0) * y0 + mira(x0);
				yn = -x0 + mira(xn);
				
				if (isNaN(xn) || isNaN(yn))
				{
					trace("Aborted", i);
					break;
				}
				
				px = ox + _zoom * xn;
				py = oy + _zoom * yn;
				
				ColorUtil.addPixel32(canvas, px, py, _color);
				x0 = xn;
				y0 = yn;
			}
			
			canvas.unlock();
			
			dispatchEvent(new Event(RENDERPASS_COMPLETE));
		}
		
		
		/**
		 * f(x) = mx + 2(1 - m)x^2 / (1 + x^2)
		 */
		private function mira(x:Number):Number
		{
			var xx:Number = x * x;
			return mu * x + (2 * (1 - mu) * xx) / (1 + xx);
		}
		
		
		private function exportImage():void
		{
			_paused = true;
			var bg:BitmapData = new BitmapData(canvas.width, canvas.height, true, 0xFFFFFFFF);
			bg.copyPixels(canvas, new Rectangle(0, 0, canvas.width, canvas.height), new Point(0, 0));
			
			var imageExporter:ImageExporter = new ImageExporter(_exportUrl, bg, _exportExt);
			imageExporter.export("mira_" + mu);
		}
		
		
		private function mouseDownListener(e:MouseEvent):void
		{
			iterations = maxIterations / 5;
			_paused = false;
			passes = 0;
			_color = interactiveColor;
		}
		
		
		private function mouseUpListener(e:MouseEvent):void
		{
			iterations = maxIterations;
			_color = finalColor;
			prepareCanvas();
		}
		
		
		private function mouseMoveListener(e:MouseEvent):void
		{
			if (e.buttonDown)
			{
				if (e.shiftKey)
				{
					a = Math.abs(0.0000001 + (stage.mouseY / stage.stageHeight) * 0.001);
				} else {
					mu = (stage.mouseX - ox) / (2 * ox);
					_zoom = 20 + (stage.mouseY - oy) * 20 / oy;
				}
				
				prepareCanvas();
			}
		}
		
		
		private function keyDownListener(e:KeyboardEvent):void
		{
			switch (e.keyCode)
			{
				case 32: /* space bar */
				_paused = true;
				renderCanvas();
				break;
				
				case 37: /* left key */
				mu -= muInc;
				prepareCanvas();
				break
				
				case 39: /* right key */
				mu += muInc;
				prepareCanvas();
				break
				
				case 38: /* up key */
				_zoom -= 1;
				prepareCanvas();
				break
				
				case 40: /* down key */
				_zoom += 1;
				prepareCanvas();
				break
				
				case 83: /* s key */
				exportImage();
			}
		}
	}
}
