/**
 * Colour Utilities
 * 
 * @author		Tom Beddard
 * @version 	0.1
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 */

package com.subblue.utils
{
	import flash.display.BitmapData;
	
	public class ColorUtil
	{
		public function ColorUtil()
		{
			
		}
		
		public static function toHex(r:uint, g:uint, b:uint, a:uint = 0xFF):uint
		{
			if (r > 0xFF) r = 0xFF;
			if (g > 0xFF) g = 0xFF;
			if (b > 0xFF) b = 0xFF;
			if (a > 0xFF) a = 0xFF;
			
			return (a << 24) | (r << 16) | (g << 8) | b;
		}
		
		public static function toRGB(c:uint):Object
		{
			return {r:red(c), g:green(c), b:blue(c), a:alpha(c)}
		}
		
		/**
		 *	Create RGB hex from hue, saturation and brightness values
		 *	hue: 0 to 360
		 *	sat: 0 to 100
		 *	val: 0 to 100
		 *	
		 *	Credit: http://www.easyrgb.com/index.php?X=MATH&H=21#text21
		 */
		public static function HSV2Hex(h:Number, s:Number, v:Number):uint
		{
			var r:Number, g:Number, b:Number, i:int;
			var j:Number, p:Number, q:Number, t:Number;
			
			// No brightness so return black
			if (v == 0) return toHex(0, 0, 0);
			
			// No saturation so return grey
			if (s == 0)
			{
				v *= 2.55;
				return toHex(int(v), int(v), int(v));
			}
			
			// Normalise so all values range 0 to 1
			h %= 360;
			h /= 60;
			s /= 100;
			v /= 100;
			
			i = int(h);
			j = h - i;
			p = v * (1 - s);
			q = v * (1 - (s * j));
			t = v * (1 - (s * (1 - j)));
			
			if (i == 0)
			{
				r = v;
				g = t;
				b = p;
			}
			else if (i == 1)
			{
				r = q;
				g = v;
				b = p;
			}
			else if (i == 2)
			{
				r = p;
				g = v;
				b = t;
			}
			else if (i == 3)
			{
				r = p;
				g = q;
				b = v;
			}
			else if (i == 4)
			{
				r = t;
				g = p;
				b = v;
			}
			else if (i == 5)
			{
				r = v;
				g = p;
				b = q;
			}
			
			return toHex(int(r * 255), int(g * 255), int(b * 255));
		}
		
		
		public static function red(c:uint):uint
		{
			return c >> 16 & 0xFF;
		}
		
		public static function green(c:uint):uint
		{
			return c >> 8 & 0xFF;
		}
		
		public static function blue(c:uint):uint
		{
			return c & 0xFF;
		}
		
		public static function alpha(c:uint):uint
		{
			return c >> 24 & 0xFF;
		}
		
		/**
		 *	Return a colour blended between two source colours with a midpoint 
		 *	p away from the first colour.
		 *	The baseColor, bc, is handy for getting the alpha blending working on a solid
		 *	color background.
		 */
		public static function blend(c1:uint, c2:uint, p:Number = 0.5, bc:uint = 0xFFFFFFFF):uint
		{
			if (p >= 1) return c2;
			if (p <= 0) return c1;
			
			var r:uint = (red(c1) + (red(c2) - red(c1)) * p);
			var g:uint = (green(c1) + (green(c2) - green(c1)) * p);
			var b:uint = (blue(c1) + (blue(c2) - blue(c1)) * p);
			var a:uint = (c1 == bc) ? alpha(c2) : Math.min(alpha(c1) + alpha(c2), 255);
			
			return toHex(r, g, b, a);
		}
		
		
		/**
		 *	Return an array of colours blended from two colours.
		 *		c1		colour 1
		 *		c2		colour 2
		 *		n		number of intermediate steps
		 *		last	include last colour - set to false if building up an array of multiple
		 *				colour blends
		 */
		public static function blendArray(c1:uint, c2:uint, n:int, last:Boolean = true):Array
		{
			var a:Array = [];
			var p:Number = 1 / (n + 1);
			
			for (var i:int = 0; i <= n; i++)
			{
				a.push(blend(c1, c2, i * p))
			}
			if (last) a.push(c2);
			
			return a;
		}
		
		
		/**
		 *	Average the colours in the given array
		 */
		public static function average(colors:Array):uint
		{
			var l:int = colors.length;
			var r:uint = 0;
			var g:uint = 0;
			var b:uint = 0;
			
			if (l == 1) return colors[0];
			
			for (var i:int = 0; i < l; i++)
			{
				r += red(colors[i]);
				g += green(colors[i]);
				b += blue(colors[i]);
			}
			return toHex(int(r / l), int(g / l), int(b / l))
		}
		
		
		/**
		 * Alpha blends with the existing pixel on the bitmap canvas
		 */
		public static function addPixel32(bmpd:BitmapData, x:uint, y:uint, color:uint, baseColor:uint = 0xFFFFFFFF):void {
			if (x > bmpd.width || y > bmpd.height) return;
			var newColor:uint = blend(bmpd.getPixel32(x, y), color, 0.5, baseColor);
			bmpd.setPixel32(x, y, newColor);
		}
		
		
		public function clone():ColorUtil {
			var clone:ColorUtil = new ColorUtil();
			return clone;
		}
		
		public function toString():String {
			return "[object ColorUtil]";
		}
		
		
	}
}